--
-- PloughTerrainDetail
-- This is the specialization for PloughTerrainDetails
--
-- @author  Stefan Geiger
-- @date  30/11/08
--
-- Copyright (C) GIANTS Software GmbH, Confidential, All Rights Reserved.

PloughTerrainDetail = {};

function PloughTerrainDetail.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Attachable, specializations);
end;

function PloughTerrainDetail:load(xmlFile)

    self.groundContactReport = SpecializationUtil.callSpecializationsFunction("groundContactReport");

    self.contactReportNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.contactReportNode#index"));
    if self.contactReportNode == nil then
        self.contactReportNode = self.components[1].node;
    end;

    self.groundReferenceThreshold = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.groundReferenceNode#threshold"), 0.2);
    self.groundReferenceNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.groundReferenceNode#index"));

    local ploughSound = getXMLString(xmlFile, "vehicle.ploughSound#file");
    if ploughSound ~= nil and ploughSound ~= "" then
        ploughSound = Utils.getFilename(ploughSound, self.baseDirectory);
        self.ploughSound = createSample("ploughSound");
        loadSample(self.ploughSound, ploughSound, false);
        self.ploughSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.ploughSound#pitchOffset"), 0);
        self.ploughSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.ploughSound#volume"), 1.0);
        self.ploughSoundEnabled = false;
    end;

    self.groundParticleSystems = {};
    local psName = "vehicle.groundParticleSystem";
    Utils.loadParticleSystem(xmlFile, self.groundParticleSystems, psName, self.components, false, nil, self.baseDirectory)
    self.groundParticleSystemActive = false;

    self.aiTerrainDetailChannel1 = g_currentMission.sowingChannel;
    self.aiTerrainDetailChannel2 = g_currentMission.cultivatorChannel;

    self.onlyActiveWhenLowered = Utils.getNoNil(getXMLBool(xmlFile, "vehicle.onlyActiveWhenLowered#value"), true);

    self.hasGroundContact = false;
    self.rotationMax = false;
    self.speedViolationMaxTime = 2500;
    self.speedViolationTimer = self.speedViolationMaxTime;
    self.ploughContactReportsActive = false;
    self.startActivationTimeout = 2000;
    self.startActivationTime = 0;
end;

function PloughTerrainDetail:delete()

    Utils.deleteParticleSystem(self.groundParticleSystems);

    if self.ploughSound ~= nil then
        delete(self.ploughSound);
    end;

    PloughTerrainDetail.removeContactReports(self);
end;

function PloughTerrainDetail:mouseEvent(posX, posY, isDown, isUp, button)
end;

function PloughTerrainDetail:keyEvent(unicode, sym, modifier, isDown)
end;

function PloughTerrainDetail:update(dt)
    if self:getIsActive() then

        local hasGroundContact = self.hasGroundContact;

        if not hasGroundContact then
            if self.groundReferenceNode ~= nil then
                local x,y,z = getWorldTranslation(self.groundReferenceNode);
                local terrainHeight = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, x, 0, z);
                if terrainHeight+self.groundReferenceThreshold >= y then
                    hasGroundContact = true;
                end;
            end;
        end;

        if hasGroundContact then

            if self.startActivationTime <= self.time then
                if not self.onlyActiveWhenLowered or self:isLowered(false) then
                    for k, cuttingArea in pairs(self.cuttingAreas) do
                        local x,y,z = getWorldTranslation(cuttingArea.start);
                        local x1,y1,z1 = getWorldTranslation(cuttingArea.width);
                        local x2,y2,z2 = getWorldTranslation(cuttingArea.height);
                        Utils.updatePloughArea(x, z, x1, z1, x2, z2);
                    end;
                end;
            end;

            if self:doCheckSpeedLimit() and self.attacherVehicle.lastSpeed*3600 > 20 then
                self.speedViolationTimer = self.speedViolationTimer - dt;
                if self.speedViolationTimer < 0 then
                    self.attacherVehicle:detachImplementByObject(self);
                end;
            else
                self.speedViolationTimer = self.speedViolationMaxTime;
            end;

            if self.ploughSound ~= nil and not self.ploughSoundEnabled then
                if self.lastSpeed*3600 > 3 and self:getIsActiveForSound() then
                    playSample(self.ploughSound, 0, self.ploughSoundVolume, 0);
                    setSamplePitch(self.ploughSound, self.ploughSoundPitchOffset);
                    self.ploughSoundEnabled = true;
                end;
            end;

            if self.lastSpeed*3600 > 5 and not self.groundParticleSystemActive then
                self.groundParticleSystemActive = true;
                Utils.setEmittingState(self.groundParticleSystems, true);
            end;

            if self.lastSpeed*3600 < 5 and self.groundParticleSystemActive then
                self.groundParticleSystemActive = false;
                Utils.setEmittingState(self.groundParticleSystems, false);
            end;

        else
            self.speedViolationTimer = self.speedViolationMaxTime;
            if self.ploughSoundEnabled then
                stopSample(self.ploughSound);
                self.ploughSoundEnabled = false;
            end;
            if self.groundParticleSystemActive then
                self.groundParticleSystemActive = false;
                Utils.setEmittingState(self.groundParticleSystems, false);
            end;
        end;


    else
        if self.groundParticleSystemActive then
            self.groundParticleSystemActive = false;
            Utils.setEmittingState(self.groundParticleSystems, false);
        end;
    end;

    if self.ploughSoundEnabled then
        if self.lastSpeed*3600 < 3 then
            stopSample(self.ploughSound);
            self.ploughSoundEnabled = false;
        end;
    end;

end;

function PloughTerrainDetail:draw()
    if math.abs(self.speedViolationTimer - self.speedViolationMaxTime) > 2 then
        g_currentMission:addWarning(g_i18n:getText("Dont_drive_to_fast") .. "\n" .. string.format(g_i18n:getText("Cruise_control_levelN"), "1", InputBinding.getButtonKeyName(InputBinding.SPEED_LEVEL1)), 0.07+0.022, 0.019+0.029);
    end;
end;

function PloughTerrainDetail:onAttach(attacherVehicle)
    PloughTerrainDetail.onActivate(self);
    PloughTerrainDetail.addContactReports(self);
    self.startActivationTime = self.time + self.startActivationTimeout;
end;

function PloughTerrainDetail:onDetach()

    if self.deactivateOnDetach then
        PloughTerrainDetail.onDeactivate(self);
        PloughTerrainDetail.removeContactReports(self);
    else
        PloughTerrainDetail.onDeactivateSounds(self);
    end;

end;

function PloughTerrainDetail:onActivate()
end;

function PloughTerrainDetail:onDeactivate()
    self.speedViolationTimer = self.speedViolationMaxTime;
    if self.groundParticleSystemActive then
        self.groundParticleSystemActive = false;
        Utils.setEmittingState(self.groundParticleSystems, false);
    end;
    PloughTerrainDetail.onDeactivateSounds(self);
end;

function PloughTerrainDetail:onDeactivateSounds()
    if self.ploughSoundEnabled then
        stopSample(self.ploughSound);
        self.ploughSoundEnabled = false;
    end;
end;

function PloughTerrainDetail:addContactReports()
    if not self.ploughContactReportsActive then
        addContactReport(self.contactReportNode, 0.0001, "groundContactReport", self);

        self.ploughContactReportsActive = true;
    end;
end;

function PloughTerrainDetail:removeContactReports()
    if self.ploughContactReportsActive then
        removeContactReport(self.contactReportNode);
        self.hasGroundContact = false;
        self.ploughContactReportsActive = false;
    end;
end;

function PloughTerrainDetail:groundContactReport(objectId, otherObjectId, isStart, normalForce, tangentialForce)

    if otherObjectId == g_currentMission.terrainRootNode then
        self.hasGroundContact = isStart or normalForce > 0 or tangentialForce > 0;
    end;

end;